<?php
/**
 * WeCodeArt Framework.
 *
 * WARNING: This file is part of the core WeCodeArt Framework. DO NOT edit this file under any circumstances.
 * Please do all modifications in the form of a child theme.
 *
 * @package 	WeCodeArt Framework
 * @subpackage 	Core\Entry\Comments
 * @copyright   Copyright (c) 2021, WeCodeArt Framework
 * @since		3.5
 * @version		5.0.0
 */

namespace WeCodeArt\Core\Entry;

defined( 'ABSPATH' ) || exit();

use WeCodeArt\Singleton;
use WeCodeArt\Markup;
use WeCodeArt\Markup\SVG;
use WeCodeArt\Markup\Input;
use WeCodeArt\Markup\Walkers\Comment as CommentWalker;

/**
 * Handles Comments Functionality
 */
class Comments {

	use Singleton;

	/**
	 * Send to Constructor
	 * @since 3.7.0
	 */
	public function init() {
		// WP Core
		add_filter( 'comment_form_fields',		[ $this, 'comment_form_fields' 		], 90 );
		add_filter( 'comment_form_defaults',	[ $this, 'comment_form_defaults' 	], 90 );
		add_filter( 'comment_reply_link',		[ $this, 'replace_reply_link_class' ] );
		add_action( 'pre_comment_on_post',  	[ $this, 'validate_cookies'			] );
	}

	/**
	 * Render Comments Info
	 *
	 * @since	3.7.3
	 * @version	5.0.0
	 *
	 * @return 	string
	 */
	public function get_comments_info( $echo = true ) {

		$comments_number = intval( get_comments_number() );

		$defaults = [
			'icon' 		=> SVG::compile( 'comments', [ 'class' => 'me-2' ] ), // Escaped with kses inside fn.
			'empty' 	=> esc_html__( 'No comments', 'wecodeart' ),
			'closed'	=> false,
			'add_one'	=> esc_html__( 'add one', 'wecodeart' ) 
		]; 

		$args = apply_filters( 'wecodeart/filter/comments/get_comments_info/args', $defaults, get_post_type() );

		$icon_html 	= '';
		$header_tx 	= '';
		if( comments_open() || $comments_number !== 0 ) {
			$icon_html = (string) $args['icon'];
			if ( 0 !== $comments_number ) {
				$header_tx = sprintf(
					_nx( '%1$s comment', '%1$s comments', $comments_number, 'comments title', 'wecodeart' ),
					number_format_i18n( $comments_number )
				);
			} else {
				$header_tx = (string) $args['empty']; 
			} 
		} else {
			if( $args['closed'] !== false && is_string( $args['closed'] ) ) $header_tx = $args['closed']; 
		} 

		// Prepare HTML output
		$output = sprintf( '%1$s %2$s', $icon_html, $header_tx );

		// Append `add comment` link
		if( comments_open() ) {
			$output .= sprintf(
				'<a class="comments__add-new float-end has-small-font-size my-2" href="#respond" rel="nofollow">%s</a>',
				(string) $args['add_one']
			); 
		}

		$output = apply_filters( 'wecodeart/filter/comments/get_comments_info/output', trim( $output ) );

		if( $echo ) {
			echo $output; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			return;
		}

		return $output; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	} 

	/**
	 * Render Comments List
	 *
	 * @since	unknown
	 * @version 4.1.61
	 *
	 * @return 	void
	 */
	public function render_comments() {
		if ( have_comments() ) {
			Markup::wrap( 'comments-list', [ [
				'tag' 	=> 'ol',
				'attrs' => [
					'id' 	=> 'comments-list',
					'class' => 'comments__list list-unstyled pl-0'
				]
			] ], function() { // Required as this to be read by theme check
				wp_list_comments( apply_filters( 'wecodeart/filter/comments/list/args', [
					'type' 		 	=> 'comment',
					'avatar_size' 	=> 60,
					'walker'		=> new CommentWalker,
				] ) );
			} );
		}
		return;
	}

	/**
	 * Render Pings List.
	 *
	 * @since	unknown
	 * @version 5.0.0
	 *
	 * @return 	void
	 */
	public function render_pings() {
		global $wp_query;

		// If have pings.
		if ( ! empty( $wp_query->comments_by_type['pings'] ) ) {
			Markup::wrap( 'pings-list', [ [
				'tag' 	=> 'ol',
				'attrs' => [
					'id' 	=> 'pings-list',
					'class' => 'comments__pings list-unstyled pl-0 pings-list'
				]
			] ], 'wp_list_comments', [ apply_filters( 'wecodeart/filter/comments/pings/args', [
				'type' 		 => 'pings',
				'short_ping' => true,
				'walker'	 => new CommentWalker,
			] ) ] );
		}
		return;
	}

	/**
	 * Render Comments Info
	 *
	 * @since	3.7.0
	 * @version	5.0.0
	 *
	 * @return	string	HTML
	 */
	public function render_meta() {  
		Markup::wrap( 'comments-info', [ [
			'tag' 	=> 'h3',
			'attrs' => [
				'id' 	=> 'comments-title',
				'class' => 'comments__headline mb-3'
			]
		] ], [ $this, 'get_comments_info' ] );
	}

	
	/**
	 * Render Coments Pagination
	 *
	 * @since 	5.0.0
	 * @version 5.0.0
	 *
	 * @return 	string|null
	 */
	public function render_pagination() {
		/**
		 * Early Break
		 */
		if( empty( get_previous_comments_link() || get_next_comments_link() ) ) return;

		Markup::wrap( 'comments-nav', [
			[
                'tag'   => 'nav',
                'attrs' => [
                    'class'         => 'comments__nav',
					'aria-label'    => esc_html__( 'Comments Navigation', 'wecodeart' ),
                ]
            ],
			[
                'tag'   => 'div',
                'attrs' => [
                    'class' => 'row pb-3'
                ]
            ]
		], function() {
			?>
            <h3 class="screen-reader-text"><?php esc_html_e( 'Comments Navigation', 'wecodeart' ); ?></h3>
			<?php

			Markup::wrap( 'comments-prev-link', [ [
				'tag' 	=> 'div',
				'attrs' => [ 
					'class' => 'col-sm-12 col-md'
				] 
			] ], 'previous_comments_link' );
	
			Markup::wrap( 'comments-next-link', [ [
				'tag' 	=> 'div',
				'attrs' => [
					'class' => 'col-sm-12 col-md text-md-end'
				] 
			] ], 'next_comments_link' );
		} );  
    }
	
	/**
	 * Render Protected
	 *
	 * @since	5.0.0
	 *
	 * @return	string	HTML
	 */
	public function render_protected() {  
		Markup::wrap( 'comment-form-protected', [ [
			'tag' 	=> 'p',
			'attrs' => [
				'class' => 'alert alert-danger shadow-soft'
			]
		] ], 'printf', [ 
			esc_html__( 'This post is password protected. Enter the password to view comments or leave a comment.', 'wecodeart' )
		] );
	}

	/**
	 * Move Comment Field Bellow Name/Email/Website.
	 *
	 * @since	unknown
	 * @version 5.0.0
	 *
	 * @param 	array $fields
	 *
	 * @return 	array
	 */
	public function comment_form_fields( $fields ) {
		$comment_field = $fields['comment'];
		$cookies_field = $fields['cookies'];
		unset( $fields['comment'], $fields['cookies'] );
		
		$fields['comment'] = $comment_field;
		$fields['cookies'] = $cookies_field;
		return $fields;
	}

	/**
	 * Filter Comment Respond Args.
	 *
	 * @since	unknown
	 * @version	5.0.0
	 *
	 * @return 	array
	 */
	public function comment_form_defaults( $defaults ) {
		// Fields escapes all the data for us. 	
		$commenter = wp_get_current_commenter();
		$req       = get_option( 'require_name_email' );

		$inputs = [
			'name' => wecodeart_input( 'floating', [
				'type' 	=> 'text',
				'label' => esc_html__( 'Name *', 'wecodeart' ),
				'attrs' => [
					'id' 	=> 'comment-author',
					'name' 	=> 'author',
					'required' 		=> ( $req ) ? 'required' : NULL,
					'size' 			=> 30,
					'maxlength' 	=> 245,
					'placeholder'	=> 'John Doe',
					'value' 		=> $commenter['comment_author']
				]
			], false ),
			'email' => wecodeart_input( 'floating', [
				'type' 	=> 'email',
				'label' => esc_html__( 'Email *', 'wecodeart' ),
				'attrs' => [
					'id' 	=> 'comment-email',
					'name' 	=> 'email',
					'required' 		=> ( $req ) ? 'required' : NULL,
					'size' 			=> 30,
					'maxlength' 	=> 100,
					'placeholder'	=> 'name@example.com',
					'value' 		=> $commenter['comment_author_email']
				]
			], false ),
			'url' => wecodeart_input( 'floating', [
				'type' 	=> 'url',
				'label' => esc_html__( 'Website', 'wecodeart' ),
				'attrs' => [
					'id' 	=> 'comment-url',
					'name' 	=> 'url',
					'size' 		 => 30, 
					'maxlength'  => 200,
					'placeholder'=> 'www.example.com',
					'value' 	 => $commenter['comment_author_url']  
				]
			], false ),
			'comment' => wecodeart_input( 'floating', [
				'type'	=> 'textarea',
				'label' => esc_html__( 'Comment *', 'wecodeart' ),
				'attrs' => [
					'id' 	=> 'comment',
					'name' 	=> 'comment',
					'rows'	=> 8,
					'cols'  => 45,
					'style'	=> 'min-height:150px;',
					'placeholder'	=> esc_html__( 'Comment *', 'wecodeart' ),
					'required'		=> ( $req ) ? 'required' : NULL,
					'aria-required'	=> 'true'
				]
			], false )
		];

		$author_name	= Markup::wrap( 'comment-author-name', [ [
			'tag' 	=> 'div',
			'attrs' => [
				'class' => 'mb-3 comment-form-author col-md-7'
			]
		] ], $inputs['name'], null, false );

		// Author Email.
		$author_email = Markup::wrap( 'comment-author-email', [ [
			'tag' 	=> 'div',
			'attrs' => [
				'class' => 'mb-3 comment-form-email col-md-7'
			]
		] ], $inputs['email'], null, false );

		// Author URL.
		$author_url	= Markup::wrap( 'comment-author-url', [ [
			'tag' 	=> 'div',
			'attrs' => [
				'class' => 'mb-3 comment-form-url col-md-7'
			]
		] ], $inputs['url'], null, false );

		// The Comment.
		$author_comment	= Markup::wrap( 'comment-field', [ [
			'tag' 	=> 'div',
			'attrs' => [
				'class' => 'mb-3 comment-form-comment'
			]
		] ], $inputs['comment'], null, false );

		// Cookies
		$cookies = false;
		$privacy_policy = get_option( 'wp_page_for_privacy_policy' );

		if( $privacy_policy && get_post_status( $privacy_policy ) === 'publish' ) {
			$cookies = Markup::wrap( 'comment-cookies', [ [
				'tag' 	=> 'div',
				'attrs' => [
					'class' => 'mb-3 comment-form-cookies'
				]
			] ], wecodeart_input( 'toggle', [
				'type'	=> 'checkbox',
				'label' => sprintf( __( 'By commenting you accept the %s.', 'wecodeart' ), sprintf(
					'<a href="%1$s">%2$s</a>',
					esc_url( get_privacy_policy_url() ),
					esc_html( get_the_title( $privacy_policy ) ) )
				),
				'attrs' => [
					'class'		=> 'form-switch',
					'id' 		=> 'comment-cookies',
					'name' 		=> 'comment-cookies',
					'required'  => ( $req ) ? 'required' : NULL,
					'aria-required'  => 'true',
				]
			], false ), null, false );
		}

		$args = [
			'format'			 	=> 'html5',
			'class_container'		=> 'wp-block-post-comments-form__wrap mt-3 mb-5',
			'class_form' 			=> 'wp-block-post-comments-form__form comment-form needs-validation',
			'title_reply' 			=> SVG::compile( 'comment-dots', [ 'class' => 'fa-2x me-2' ] ) . esc_html__( 'Speak Your Mind', 'wecodeart' ),
			'title_reply_before' 	=> '<h3 id="reply-title" class="wp-block-post-comments-form__headline mb-3">',
			'title_reply_after'  	=> '</h3>',
			'cancel_reply_before'  => '<span class="has-small-font-size my-2 float-end">',
			'cancel_reply_after'   => '</span>',
			'cancel_reply_link'    => esc_html__( 'Cancel reply', 'wecodeart' ),
			'comment_field' 		=> $author_comment,
			'comment_notes_before' 	=> Markup::wrap( 'comment-notes-before', [ [
				'tag' 	=> 'div',
				'attrs' => [
					'class' => 'mb-3 comment-form-notes'
				]
			] ], function() use ( $req ) {
				$string = esc_html__( 'Your email address will not be published.', 'wecodeart' );
				if( $req ) {
					$string .= ' ' . sprintf( esc_html__( 'Required fiels are marked "%s".', 'wecodeart' ), 
						'<span class="required">*</span>' 
					);
				}
				echo wp_kses_post( $string );
			}, [], false ),
			'submit_field'	=> '<div class="mb-3 comment-form-submit">%1$s %2$s</div>',
			'submit_button'	=> '<button name="%1$s" type="submit" id="%2$s" class="%3$s">' . SVG::compile( 'comment-dots', [
				'class' => 'me-1'
			] ) . '<span>%4$s</span></button>',
			'class_submit'	=> 'btn btn-outline-dark',
			'fields' => [
				'author' 	=> $author_name,
				'email'  	=> $author_email,
				'url'    	=> $author_url,
				'cookies'	=> $cookies,
			]
		];

		return wp_parse_args( $args, $defaults );
	}

	/**
	 * Replace Comment Reply Button class.
	 *
	 * @since	unknown
	 * @version 5.0.0
	 *
	 * @param 	string $html
	 *
	 * @return 	string
	 */
	public function replace_reply_link_class( $html ) {
		$search 	= '/' . preg_quote( "class='comment-reply-link'", '/' ) . '/';
		$replace 	= 'class="comment-reply-link btn btn-dark btn-sm"';
		return preg_replace( $search, $replace, $html, 1 );
	}

	/**
	 * Validate Cookie field.
	 *
	 * @since	5.0.0
	 * @since	5.0.5
	 *
	 * @return 	void
	 */
	public function validate_cookies() {
		$privacy_policy = get_option( 'wp_page_for_privacy_policy' );

		if( get_post_status( $privacy_policy ) !== 'publish' || is_user_logged_in() === false ) return;

		if( ! filter_input( INPUT_POST, 'comment-cookies' ) ) {
			wp_die( sprintf( esc_html__( 'You must accept %s to comment!', 'wecodeart' ), sprintf(
				'<a href="%1$s" target="_blank">%2$s</a>',
				esc_url( get_privacy_policy_url() ),
				esc_html( get_the_title( $privacy_policy ) )
			) ) );
		}
	}
}